from python_helper import ObjectHelper, StringHelper, SettingHelper, Constant, log, ReflectionHelper, Test

# LOG_HELPER_SETTINGS = {
#     log.LOG : True,
#     log.SUCCESS : True,
#     log.SETTING : True,
#     log.DEBUG : True,
#     log.WARNING : True,
#     log.WRAPPER : True,
#     log.FAILURE : True,
#     log.ERROR : True,
    # log.TEST : False
# }

LOG_HELPER_SETTINGS = {
    log.LOG : False,
    log.SUCCESS : False,
    log.SETTING : False,
    log.DEBUG : False,
    log.WARNING : False,
    log.WRAPPER : False,
    log.FAILURE : False,
    log.ERROR : False,
    log.TEST : False
}

TEST_SETTINGS = {}

@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def isNotMethodInstance_withSuccess() :
    # Arrange
    class MyObject:
        def __init__(self, myAttribute) :
            self.myAttribute = myAttribute
        def myMethod(self):
            return self.myAttribute
    class MyOtherObject:
        def __init__(self, myAttribute) :
            self.myAttribute = myAttribute
            self.myHiddenAttribute = 2
        def myMethod(self):
            return self.myAttribute
    myObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyObject)
    myOtherObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyObject)
    MY_ATTRIBUTE_NAME = 'myAttribute'
    MY_METHOD_NAME = 'myMethod'
    MY_NOT_EXISTING_ATTRIBUTE_OR_METHOD_NAME = 'myNotExistingMethodOrAtrributeName'

    # Act
    attributeOrMethodNameList = ReflectionHelper.getAttributeOrMethodNameList(myObject.__class__)
    attributeNameList = ReflectionHelper.getAttributeNameList(myObject.__class__)
    methodNameList = ReflectionHelper.getMethodNameList(myObject.__class__)
    myAttribute = getattr(myObject, MY_ATTRIBUTE_NAME)
    myMethod = getattr(myObject, 'myMethod')

    # Assert
    assert [MY_ATTRIBUTE_NAME, MY_METHOD_NAME] == attributeOrMethodNameList
    assert [MY_ATTRIBUTE_NAME] == attributeNameList
    assert [MY_METHOD_NAME] == methodNameList
    assert ReflectionHelper.isNotMethodInstance(myAttribute)
    assert not ReflectionHelper.isNotMethodInstance(myMethod)
    assert ReflectionHelper.isNotMethodInstance(ReflectionHelper.getAttributeOrMethod(myObject, MY_NOT_EXISTING_ATTRIBUTE_OR_METHOD_NAME))

    assert ReflectionHelper.isNotMethod(myObject, MY_ATTRIBUTE_NAME)
    assert not ReflectionHelper.isNotMethod(myObject, MY_METHOD_NAME)
    assert ReflectionHelper.isNotMethod(myObject, MY_NOT_EXISTING_ATTRIBUTE_OR_METHOD_NAME)

    assert ReflectionHelper.isMethodInstance(myMethod)
    assert not ReflectionHelper.isMethodInstance(myAttribute)
    assert not ReflectionHelper.isMethodInstance(ReflectionHelper.getAttributeOrMethod(myObject, MY_NOT_EXISTING_ATTRIBUTE_OR_METHOD_NAME))

    assert not ReflectionHelper.isMethod(myObject, MY_ATTRIBUTE_NAME)
    assert ReflectionHelper.isMethod(myObject, MY_METHOD_NAME)
    assert not ReflectionHelper.isMethod(myObject, MY_NOT_EXISTING_ATTRIBUTE_OR_METHOD_NAME)

@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def instanciateItWithNoArgsConstructor_withSuccess() :
    # Arrange
    class MyObject:
        def __init__(self, myAttribute) :
            self.myAttribute = myAttribute
    class MyOtherObject:
        def __init__(self, myAttribute, myArg, myKeyword=None) :
            self.myAttribute = myAttribute
            self.myHiddenAttribute = 2
    # from python_framework import Enum, EnumItem
    # @Enum(associateReturnsTo='number')
    # class WeekDayEnumeration :
    #     MONDAY = EnumItem(number=0)
    #     TUESDAY = EnumItem(number=1)
    #     WEDNESDAY = EnumItem(number=2)
    #     THURSDAY = EnumItem(number=3)
    #     FRIDAY = EnumItem(number=4)
    #     SATURDAY = EnumItem(number=5)
    #     SUNDAY = EnumItem(number=6)
    # WeekDay = WeekDayEnumeration()
    # class MyEnumOtherObject:
    #     def __init__(self, myAttribute=None, enum=None, myKeyword=None) :
    #         self.myAttribute = myAttribute
    #         self.myEnum = WeekDay.map(enum)
    # myEnumOtherObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyOtherObject)

    # Act
    myObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyObject)
    myOtherObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyOtherObject)

    # Assert
    assert ObjectHelper.isNotNone(myObject)
    assert ObjectHelper.isNone(myObject.myAttribute)
    assert ObjectHelper.isNotNone(myOtherObject)
    assert ObjectHelper.isNone(myOtherObject.myAttribute)
    assert 2 == myOtherObject.myHiddenAttribute

@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def overrideSignatures_withSuccess() :
    # Arrange
    class MyObject:
        def myMethod(self, something):
            return something

    def myWrapper(myFunction,*wrapperAgrs,**wrapperKwargs):
        def myInBetweenWrapperFunction(*agrs,**kwargs) :
            return myFunction(*agrs,**kwargs)
        ReflectionHelper.overrideSignatures(myInBetweenWrapperFunction, myFunction)
        return myInBetweenWrapperFunction

    def myWrapperWithoutOverride(myFunction,*wrapperAgrs,**wrapperKwargs):
        def myInBetweenWrapperFunction(*agrs,**kwargs) :
            return myFunction(*agrs,**kwargs)
        return myInBetweenWrapperFunction

    OVERRIDED = '__overrided'
    NOT_OVERRIDED = '__not_overrided'

    @myWrapper
    def myOverridedFunction(arg) :
        return f'{arg}{OVERRIDED}'

    @myWrapperWithoutOverride
    def myNotOverridedFunction(arg) :
        return f'{arg}{NOT_OVERRIDED}'

    ABCD = 'abcd'
    exception = None

    # Act
    myObject = ReflectionHelper.instanciateItWithNoArgsConstructor(MyObject)
    ReflectionHelper.setAttributeOrMethod(myObject, myOverridedFunction.__name__, myOverridedFunction)
    ReflectionHelper.setAttributeOrMethod(myObject, myNotOverridedFunction.__name__, myNotOverridedFunction)
    try :
        myObject.myNotOverridedFunction('any arg')
    except Exception as e :
        exception = e

    # Assert
    assert myObject.myMethod(True)
    assert not myObject.myMethod(False)
    assert ObjectHelper.isNotNone(myOverridedFunction(True))
    assert ObjectHelper.isNotNone(myOverridedFunction(False))
    assert ObjectHelper.isNotNone(myOverridedFunction(ABCD))
    assert ObjectHelper.isNotNone(myNotOverridedFunction(True))
    assert ObjectHelper.isNotNone(myNotOverridedFunction(False))
    assert ObjectHelper.isNotNone(myNotOverridedFunction(ABCD))
    assert ObjectHelper.isNotNone(myObject.myOverridedFunction(True))
    assert ObjectHelper.isNotNone(myObject.myOverridedFunction(False))
    assert ObjectHelper.isNotNone(myObject.myOverridedFunction(ABCD))
    assert myOverridedFunction(True) == myObject.myOverridedFunction(True)
    assert myOverridedFunction(False) == myObject.myOverridedFunction(False)
    assert myOverridedFunction(ABCD) == myObject.myOverridedFunction(ABCD)
    assert f'{True}{OVERRIDED}' == myObject.myOverridedFunction(True)
    assert f'{False}{OVERRIDED}' == myObject.myOverridedFunction(False)
    assert f'{ABCD}{OVERRIDED}' == myObject.myOverridedFunction(ABCD)
    assert ObjectHelper.isNotNone(exception)
    assert "'MyObject' object has no attribute 'myNotOverridedFunction'" == str(exception)
    assert ObjectHelper.isNotNone(myObject.myInBetweenWrapperFunction(True))
    assert ObjectHelper.isNotNone(myObject.myInBetweenWrapperFunction(False))
    assert ObjectHelper.isNotNone(myObject.myInBetweenWrapperFunction(ABCD))
    assert myNotOverridedFunction(True) == myObject.myInBetweenWrapperFunction(True)
    assert myNotOverridedFunction(False) == myObject.myInBetweenWrapperFunction(False)
    assert myNotOverridedFunction(ABCD) == myObject.myInBetweenWrapperFunction(ABCD)
    assert f'{True}{NOT_OVERRIDED}' == myObject.myInBetweenWrapperFunction(True)
    assert f'{False}{NOT_OVERRIDED}' == myObject.myInBetweenWrapperFunction(False)
    assert f'{ABCD}{NOT_OVERRIDED}' == myObject.myInBetweenWrapperFunction(ABCD)

@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def getArgsOrder_withSuccess() :
    # Arrange
    class MyClass :
        def __init__(
            self,
            attributeValue,
            otherAttributeValue,
            otherAttributeValueAgain,
            someOtherAttributeValue,
            otherAttributeValueOnceAgain
        ):
            self.attributeKey = attributeValue
            self.otherAttributeKey = otherAttributeValue
            self.otherAttributeKeyAgain = otherAttributeValueAgain
            self.someOtherAttributeKey = someOtherAttributeValue
            self.otherAttributeKeyOnceAgain = otherAttributeValueOnceAgain
    class MyClassWithoutAttributes :
        def __init__(self):
            ...
    class MyClassWithMoreAttributesThanArgs :
        def __init__(self,attributeValue):
            self.attributeKey = attributeValue
            self.otherAttributeKey = None
            self.otherAttributeKeyAgain = None
            self.someOtherAttributeKey = None
            self.otherAttributeKeyOnceAgain = None
    class MyClassWithLessAttributesThanArgs :
        def __init__(
            self,
            attributeValue,
            otherAttributeValue,
            otherAttributeValueAgain,
            someOtherAttributeValue,
            otherAttributeValueOnceAgain
        ):
            self.attributeKey = attributeValue
    class MyClassWithoutInitMethod :
        attributeKey = None
        otherAttributeKey = None
        otherAttributeKeyAgain = None
        someOtherAttributeKey = None
        otherAttributeKeyOnceAgain = None

    # Act
    myClassArgsOrder = ReflectionHelper.getArgsOrder(MyClass)
    myClassWithoutAttributesArgsOrder = ReflectionHelper.getArgsOrder(MyClassWithoutAttributes)
    myClassWithMoreAttributesThanArgsArgsOrder = ReflectionHelper.getArgsOrder(MyClassWithMoreAttributesThanArgs)
    myClassWithLessAttributesThanArgsArgsOrder = ReflectionHelper.getArgsOrder(MyClassWithLessAttributesThanArgs)
    myClassWithoutInitMethodArgsOrder = ReflectionHelper.getArgsOrder(MyClassWithoutInitMethod)

    # Assert
    assert [
        'attributeKey',
        'otherAttributeKey',
        'otherAttributeKeyAgain',
        'someOtherAttributeKey',
        'otherAttributeKeyOnceAgain'
    ] == myClassArgsOrder
    assert [] == myClassWithoutAttributesArgsOrder
    assert [
        'attributeKey'
    ] == myClassWithLessAttributesThanArgsArgsOrder
    assert [
        'attributeKey'
    ] == myClassWithMoreAttributesThanArgsArgsOrder
    assert [] == myClassWithoutInitMethodArgsOrder


@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def getAttributeOrMethodByNamePath():
    #arrange
    class MyClass :
        def __init__(self, value):
            self.myAttribute = value
    FINAL_VALUE = 3
    DEFAULT_VALUE = 7
    instance = MyClass(MyClass(MyClass(MyClass(3))))

    #act
    finalValueToAssert = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.myAttribute.myAttribute.myAttribute', muteLogs=False, default=None)

    mustBeNone = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.myAttribute.myAttribute.myAttribute.myAttribute', muteLogs=False, default=None)
    mustBeDefault = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.myAttribute.myAttribute.myAttribute.myAttribute', muteLogs=False, default=DEFAULT_VALUE)

    mustBeNoneByEndingOnDot = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.', muteLogs=False, default=None)
    mustBeDefaultByEndingOnDot = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.', muteLogs=False, default=DEFAULT_VALUE)

    mustBeNoneByStartingOnDot = ReflectionHelper.getAttributeOrMethodByNamePath(instance, '.myAttribute', muteLogs=False, default=None)
    mustBeDefaultByStartingOnDot = ReflectionHelper.getAttributeOrMethodByNamePath(instance, '.myAttribute', muteLogs=False, default=DEFAULT_VALUE)

    hereWeGo = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute..myAttribute', muteLogs=False, default=None)
    hereWeGoDefault = ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute..myAttribute', muteLogs=False, default=DEFAULT_VALUE)

    #assert
    assert FINAL_VALUE == finalValueToAssert, f'{FINAL_VALUE} == {finalValueToAssert}'

    assert ObjectHelper.isNone(mustBeNone)
    assert DEFAULT_VALUE == mustBeDefault, f'{DEFAULT_VALUE} == {mustBeDefault}'

    assert ObjectHelper.isNone(mustBeNoneByEndingOnDot)
    assert DEFAULT_VALUE == mustBeDefaultByEndingOnDot

    assert ObjectHelper.isNone(mustBeNoneByStartingOnDot)
    assert DEFAULT_VALUE == mustBeDefaultByStartingOnDot

    assert ObjectHelper.isNone(hereWeGo)
    assert DEFAULT_VALUE == hereWeGoDefault

    assert ObjectHelper.isNotNone(instance.myAttribute.myAttribute)
    assert ObjectHelper.equals(instance.myAttribute.myAttribute, ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.myAttribute', muteLogs=False, default=None))
    assert ObjectHelper.isNotNone(instance.myAttribute)
    assert not ObjectHelper.equals(instance.myAttribute, ReflectionHelper.getAttributeOrMethodByNamePath(instance, 'myAttribute.myAttribute', muteLogs=False, default=None))


@Test(
    environmentVariables={
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        **LOG_HELPER_SETTINGS
    },
    **TEST_SETTINGS
)
def getName_manyCases() :
    # arrange
    def functionMine() :
        return None
    class Me:
        an = 2
        def __init__(self) :
            self.na = 'yey'
        def methodIsMine() :
            return True

    # act
    functionNameToAsert = ReflectionHelper.getName(functionMine)
    instanceNameToAsert = ReflectionHelper.getName(Me())
    classNameToAsert = ReflectionHelper.getName(Me)
    methodNameToAsert = ReflectionHelper.getName(Me().methodIsMine)
    attributeNameToAsert = ReflectionHelper.getName(Me().an)
    staticAttributeNameToAsert = ReflectionHelper.getName(Me().na)

    class_functionNameToAsert = ReflectionHelper.getClassName(functionMine)
    class_instanceNameToAsert = ReflectionHelper.getClassName(Me())
    class_classNameToAsert = ReflectionHelper.getClassName(Me)
    class_methodNameToAsert = ReflectionHelper.getClassName(Me().methodIsMine)
    class_attributeNameToAsert = ReflectionHelper.getClassName(Me().an)
    class_staticAttributeNameToAsert = ReflectionHelper.getClassName(Me().na)

    type_functionNameToAsert = ReflectionHelper.getName(type(functionMine))
    type_instanceNameToAsert = ReflectionHelper.getName(type(Me()))
    type_classNameToAsert = ReflectionHelper.getName(type(Me))
    type_methodNameToAsert = ReflectionHelper.getName(type(Me().methodIsMine))
    type_attributeNameToAsert = ReflectionHelper.getName(type(Me().an))
    type_staticAttributeNameToAsert = ReflectionHelper.getName(type(Me().na))

    type_class_functionNameToAsert = ReflectionHelper.getClassName(type(functionMine))
    type_class_instanceNameToAsert = ReflectionHelper.getClassName(type(Me()))
    type_class_classNameToAsert = ReflectionHelper.getClassName(type(Me))
    type_class_methodNameToAsert = ReflectionHelper.getClassName(type(Me().methodIsMine))
    type_class_attributeNameToAsert = ReflectionHelper.getClassName(type(Me().an))
    type_class_staticAttributeNameToAsert = ReflectionHelper.getClassName(type(Me().na))

    None_functionNameToAsert = ReflectionHelper.getName(None)
    None_instanceNameToAsert = ReflectionHelper.getName(None)
    None_classNameToAsert = ReflectionHelper.getName(None)
    None_methodNameToAsert = ReflectionHelper.getName(None)
    None_attributeNameToAsert = ReflectionHelper.getName(None)
    None_staticAttributeNameToAsert = ReflectionHelper.getName(None)

    None_class_functionNameToAsert = ReflectionHelper.getClassName(None)
    None_class_instanceNameToAsert = ReflectionHelper.getClassName(None)
    None_class_classNameToAsert = ReflectionHelper.getClassName(None)
    None_class_methodNameToAsert = ReflectionHelper.getClassName(None)
    None_class_attributeNameToAsert = ReflectionHelper.getClassName(None)
    None_class_staticAttributeNameToAsert = ReflectionHelper.getClassName(None)

    parent_functionNameToAsert = ReflectionHelper.getParentClass(functionMine)
    parent_instanceNameToAsert = ReflectionHelper.getParentClass(Me())
    parent_classNameToAsert = ReflectionHelper.getParentClass(Me)
    actual_parent_methodNameToAsert = Me()
    parent_methodNameToAsert = ReflectionHelper.getParentClass(actual_parent_methodNameToAsert.methodIsMine)
    parent_attributeNameToAsert = ReflectionHelper.getParentClass(Me().an)
    parent_staticAttributeNameToAsert = ReflectionHelper.getParentClass(Me().na)
    parent_testFunctionNameToAsert = ReflectionHelper.getParentClass(getName_manyCases)
    # print(
    #     functionNameToAsert,
    #     instanceNameToAsert,
    #     classNameToAsert,
    #     methodNameToAsert,
    #     attributeNameToAsert,
    #     staticAttributeNameToAsert
    # )
    # print(
    #     class_functionNameToAsert,
    #     class_instanceNameToAsert,
    #     class_classNameToAsert,
    #     class_methodNameToAsert,
    #     class_attributeNameToAsert,
    #     class_staticAttributeNameToAsert
    # )
    # print(
    #     type_functionNameToAsert,
    #     type_instanceNameToAsert,
    #     type_classNameToAsert,
    #     type_methodNameToAsert,
    #     type_attributeNameToAsert,
    #     type_staticAttributeNameToAsert
    # )
    # print(
    #     type_class_functionNameToAsert,
    #     type_class_instanceNameToAsert,
    #     type_class_classNameToAsert,
    #     type_class_methodNameToAsert,
    #     type_class_attributeNameToAsert,
    #     type_class_staticAttributeNameToAsert
    # )
    # print(
    #     None_functionNameToAsert,
    #     None_instanceNameToAsert,
    #     None_classNameToAsert,
    #     None_methodNameToAsert,
    #     None_attributeNameToAsert,
    #     None_staticAttributeNameToAsert
    # )
    # print(
    #     None_class_functionNameToAsert,
    #     None_class_instanceNameToAsert,
    #     None_class_classNameToAsert,
    #     None_class_methodNameToAsert,
    #     None_class_attributeNameToAsert,
    #     None_class_staticAttributeNameToAsert
    # )
    # assert
    assert 'functionMine' == functionNameToAsert
    assert '(undefined)' == instanceNameToAsert
    assert 'Me' == classNameToAsert
    assert 'methodIsMine' == methodNameToAsert
    assert '(undefined)' == attributeNameToAsert
    assert '(undefined)' == staticAttributeNameToAsert

    assert 'function' == class_functionNameToAsert
    assert 'Me' == class_instanceNameToAsert
    assert 'type' == class_classNameToAsert
    assert 'method' == class_methodNameToAsert
    assert 'int' == class_attributeNameToAsert
    assert 'str' == class_staticAttributeNameToAsert

    assert 'function' == type_functionNameToAsert
    assert 'Me' == type_instanceNameToAsert
    assert 'type' == type_classNameToAsert
    assert 'method' == type_methodNameToAsert
    assert 'int' == type_attributeNameToAsert
    assert 'str' == type_staticAttributeNameToAsert

    assert 'type' == type_class_functionNameToAsert
    assert 'type' == type_class_instanceNameToAsert
    assert 'type' == type_class_classNameToAsert
    assert 'type' == type_class_methodNameToAsert
    assert 'type' == type_class_attributeNameToAsert
    assert 'type' == type_class_staticAttributeNameToAsert

    assert '(undefined)' == None_functionNameToAsert
    assert '(undefined)' == None_instanceNameToAsert
    assert '(undefined)' == None_classNameToAsert
    assert '(undefined)' == None_methodNameToAsert
    assert '(undefined)' == None_attributeNameToAsert
    assert '(undefined)' == None_staticAttributeNameToAsert

    assert '(undefined)' == None_class_functionNameToAsert
    assert '(undefined)' == None_class_instanceNameToAsert
    assert '(undefined)' == None_class_classNameToAsert
    assert '(undefined)' == None_class_methodNameToAsert
    assert '(undefined)' == None_class_attributeNameToAsert
    assert '(undefined)' == None_class_staticAttributeNameToAsert

    assert None == parent_functionNameToAsert, parent_functionNameToAsert
    assert None == parent_instanceNameToAsert, parent_instanceNameToAsert
    assert None == parent_classNameToAsert, parent_classNameToAsert
    assert actual_parent_methodNameToAsert.__class__ == parent_methodNameToAsert, f'{actual_parent_methodNameToAsert} == {parent_methodNameToAsert}'
    assert None == parent_attributeNameToAsert, parent_attributeNameToAsert
    assert None == parent_staticAttributeNameToAsert, parent_staticAttributeNameToAsert
    assert parent_testFunctionNameToAsert == ReflectionHelper.getParentClass(getName_manyCases), f'{parent_testFunctionNameToAsert} == {ReflectionHelper.getName(getName_manyCases)}'



from python_helper import log, SettingHelper, StringHelper, EnvironmentVariable, EnvironmentHelper, ObjectHelper, Test

class MyClass :
    def myMethod(self):
        self.me = 1
def myFunction(a):
    return str(a) + 'b'
OPTIONAL_EXCEPTION_LOG_TYPES = [log.log, log.debug, log.warning, log.test, log.wrapper]
DICTIONARY_INSTANCE = {
    'my_none_value' : None,
    'my_none_value-as_string' : 'None',
    'string_key_as_string' : 'value',
    'integer_key_as_string' : 12345678901,
    10987654321 : 12345678901,
    'bool_key_as_string' : True,
    False : True,
    'float_key_as_string' : 12345.678901,
    109876.54321 : 12345.678901,
    'list_key_as_string' : [
        'my',
        'list',
        'elements'
    ],
    'list_of_list_key_as_string' : [
        [
            'my',
            'first',
            'list',
            False,
            12345.678901
        ],
        [
            'my',
            'second',
            'list',
            True,
            10987654321
        ]
    ],
    'set_of_string_key_as_string' : {
        'a',
        'True',
        '2',
        '3.4',
        True,
        2,
        2.4
    }
}

TEST_SETTINGS = {}

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : True,
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT
    },
    **TEST_SETTINGS
)
def mustLogWithColors() :
    # Arrange
    noExceptionThrown = 'exception not thrown'
    someLogMessage = 'some log message'
    someExceptionMessage = 'some exception message'
    someInnerExceptionMessage = 'some inner exception message'
    exception = None
    someExceptionMessageWithStackTrace = f'{someExceptionMessage} with stacktrace'
    someExceptionMessageWithoutStackTrace = f'{someExceptionMessage} without stacktrace'
    def controlableException(logType, muteStackTrace=False) :
        try :
            raise Exception(someExceptionMessageWithoutStackTrace if muteStackTrace else someExceptionMessageWithStackTrace)
        except Exception as exception :
            if logType in OPTIONAL_EXCEPTION_LOG_TYPES :
                logType(logType, someLogMessage, exception=exception, muteStackTrace=muteStackTrace)
            else :
                logType(logType, someLogMessage, exception, muteStackTrace=muteStackTrace)

    # Act
    log.info(log.info, someLogMessage)
    log.status(log.status, someLogMessage)
    log.success(log.success, someLogMessage)
    log.setting(log.setting, someLogMessage)
    log.debug(log.debug, someLogMessage)
    log.warning(log.warning, someLogMessage)

    controlableException(log.log)
    controlableException(log.debug)
    controlableException(log.warning)
    controlableException(log.wrapper)
    controlableException(log.failure)
    controlableException(log.error)
    controlableException(log.test)

    controlableException(log.log, muteStackTrace=True)
    controlableException(log.debug, muteStackTrace=True)
    controlableException(log.warning, muteStackTrace=True)
    controlableException(log.wrapper, muteStackTrace=True)
    controlableException(log.failure, muteStackTrace=True)
    controlableException(log.error, muteStackTrace=True)
    controlableException(log.test, muteStackTrace=True)

    log.log(log.log, someLogMessage, None)
    log.debug(log.debug, someLogMessage, None)
    log.warning(log.warning, someLogMessage, None)
    log.wrapper(log.wrapper, noExceptionThrown, None)
    log.failure(log.failure, noExceptionThrown, None)
    log.error(log.error, noExceptionThrown, None)
    log.test(log.test, someLogMessage, None)

    # Assert
    assert SettingHelper.LOCAL_ENVIRONMENT == EnvironmentHelper.get(SettingHelper.ACTIVE_ENVIRONMENT)
    assert SettingHelper.activeEnvironmentIsLocal()

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : True,
        log.ENABLE_LOGS_WITH_COLORS : False,
        SettingHelper.ACTIVE_ENVIRONMENT : 'my environment'
    },
    **TEST_SETTINGS
)
def mustLogWithoutColors() :
    # Arrange
    noExceptionThrown = 'exception not thrown'
    someLogMessage = 'some log message'
    someExceptionMessage = 'some exception message'
    someInnerExceptionMessage = 'some inner exception message'
    exception = None
    someExceptionMessageWithStackTrace = f'{someExceptionMessage} with stacktrace'
    someExceptionMessageWithoutStackTrace = f'{someExceptionMessage} without stacktrace'
    def controlableException(logType, muteStackTrace=False) :
        try :
            raise Exception(someExceptionMessageWithoutStackTrace if muteStackTrace else someExceptionMessageWithStackTrace)
        except Exception as exception :
            if logType in OPTIONAL_EXCEPTION_LOG_TYPES :
                logType(logType, someLogMessage, exception=exception, muteStackTrace=muteStackTrace)
            else :
                logType(logType, someLogMessage, exception, muteStackTrace=muteStackTrace)

    # Act
    log.info(log.info, someLogMessage)
    log.status(log.status, someLogMessage)
    log.success(log.success, someLogMessage)
    log.setting(log.setting, someLogMessage)
    log.debug(log.debug, someLogMessage)
    log.warning(log.warning, someLogMessage)

    controlableException(log.log)
    controlableException(log.debug)
    controlableException(log.warning)
    controlableException(log.wrapper)
    controlableException(log.failure)
    controlableException(log.error)
    controlableException(log.test)

    controlableException(log.log, muteStackTrace=True)
    controlableException(log.debug, muteStackTrace=True)
    controlableException(log.warning, muteStackTrace=True)
    controlableException(log.wrapper, muteStackTrace=True)
    controlableException(log.failure, muteStackTrace=True)
    controlableException(log.error, muteStackTrace=True)
    controlableException(log.test, muteStackTrace=True)

    log.log(log.log, someLogMessage, None)
    log.debug(log.debug, someLogMessage, None)
    log.warning(log.warning, someLogMessage, None)
    log.wrapper(log.wrapper, noExceptionThrown, None)
    log.failure(log.failure, noExceptionThrown, None)
    log.error(log.error, noExceptionThrown, None)
    log.test(log.test, someLogMessage, None)

    # Assert
    assert 'my environment' == EnvironmentHelper.get(SettingHelper.ACTIVE_ENVIRONMENT)

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : False,
        log.ENABLE_LOGS_WITH_COLORS : None,
        SettingHelper.ACTIVE_ENVIRONMENT : None
    },
    **TEST_SETTINGS
)
def mustLogWithoutColorsAsWell() :
    # Arrange
    noExceptionThrown = 'exception not thrown'
    someLogMessage = 'some log message'
    someExceptionMessage = 'some exception message'
    someInnerExceptionMessage = 'some inner exception message'
    exception = None
    someExceptionMessageWithStackTrace = f'{someExceptionMessage} with stacktrace'
    someExceptionMessageWithoutStackTrace = f'{someExceptionMessage} without stacktrace'
    def controlableException(logType, muteStackTrace=False) :
        try :
            raise Exception(someExceptionMessageWithoutStackTrace if muteStackTrace else someExceptionMessageWithStackTrace)
        except Exception as exception :
            if logType in OPTIONAL_EXCEPTION_LOG_TYPES :
                logType(logType, someLogMessage, exception=exception, muteStackTrace=muteStackTrace)
            else :
                logType(logType, someLogMessage, exception, muteStackTrace=muteStackTrace)

    # Act
    log.info(log.info, someLogMessage)
    log.status(log.status, someLogMessage)
    log.success(log.success, someLogMessage)
    log.setting(log.setting, someLogMessage)
    log.debug(log.debug, someLogMessage)
    log.warning(log.warning, someLogMessage)

    controlableException(log.log)
    controlableException(log.debug)
    controlableException(log.warning)
    controlableException(log.wrapper)
    controlableException(log.failure)
    controlableException(log.error)
    controlableException(log.test)

    controlableException(log.log, muteStackTrace=True)
    controlableException(log.debug, muteStackTrace=True)
    controlableException(log.warning, muteStackTrace=True)
    controlableException(log.wrapper, muteStackTrace=True)
    controlableException(log.failure, muteStackTrace=True)
    controlableException(log.error, muteStackTrace=True)
    controlableException(log.test, muteStackTrace=True)

    log.log(log.log, noExceptionThrown, None)
    log.debug(log.debug, noExceptionThrown, None)
    log.warning(log.warning, noExceptionThrown, None)
    log.wrapper(log.wrapper, noExceptionThrown, None)
    log.failure(log.failure, noExceptionThrown, None)
    log.error(log.error, noExceptionThrown, None)
    log.test(log.test, noExceptionThrown, None)

    # Assert
    assert True == SettingHelper.activeEnvironmentIsDefault()
    assert SettingHelper.DEFAULT_ENVIRONMENT == EnvironmentHelper.get(SettingHelper.ACTIVE_ENVIRONMENT)
    assert SettingHelper.DEFAULT_ENVIRONMENT == SettingHelper.getActiveEnvironment()

@Test(
    environmentVariables={
        log.LOG : False,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : False,
        log.WARNING : False,
        log.WRAPPER : False,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : False,
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT,
        'SOME_PARTICULAR_SETTING' : '"some value"'
    },
    **TEST_SETTINGS
)
def mustLogEnvironmentSettings() :
    # Arrange

    # Act
    SettingHelper.logEnvironmentSettings()

    # Assert
    assert SettingHelper.LOCAL_ENVIRONMENT == EnvironmentHelper.get(SettingHelper.ACTIVE_ENVIRONMENT)
    assert SettingHelper.LOCAL_ENVIRONMENT == SettingHelper.getActiveEnvironment()
    assert SettingHelper.activeEnvironmentIsLocal()
    assert "some value" == EnvironmentHelper.get('SOME_PARTICULAR_SETTING')

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : False,
        log.ENABLE_LOGS_WITH_COLORS : False,
        SettingHelper.ACTIVE_ENVIRONMENT : None
    },
    **TEST_SETTINGS
)
def mustLogPretyPythonWithoutColors() :
    # Arrange
    dictionaryInstance = {**{}, **DICTIONARY_INSTANCE}
    exception = None

    # Act
    try :
        log.prettyPython(mustLogPretyPythonWithoutColors, 'prettyPythonWithoutColors', dictionaryInstance)
    except Exception as e :
        log.failure(mustLogPretyPythonWithoutColors, 'Failed to log prety python in this method call', e)
        exception = e

    # Assert
    assert ObjectHelper.isNone(exception)

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : False,
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT
    },
    **TEST_SETTINGS
)
def mustLogPretyPythonWithColors() :
    # Arrange
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass}): {type(MyClass)}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass}).__name__: {type(MyClass).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass().myMethod}): {type(MyClass().myMethod)}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass().myMethod}).__name__: {type(MyClass().myMethod).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({myFunction}): {type(myFunction)}')
    # log.log(mustLogPretyPythonWithColors, f'type({myFunction}).__name__: {type(myFunction).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({log}): {type(log)}')
    # log.log(mustLogPretyPythonWithColors, f'type({log}).__name__: {type(log).__name__}')
    dictionaryInstance = {
        **{
            'class' : MyClass,
            'method' : MyClass().myMethod,
            'value' : MyClass().myMethod(),
            'function' : myFunction,
            'otherValue' : myFunction(1.1),
            'module' : log
        },
        **DICTIONARY_INSTANCE
    }
    exception = None

    # Act
    try :
        log.prettyPython(mustLogPretyPythonWithColors, 'prettyPython', dictionaryInstance)
    except Exception as e :
        log.failure(mustLogPretyPythonWithColors, 'Failed to log prety python in this method call', e)
        exception = e

    # Assert
    assert ObjectHelper.isNone(exception)

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : False,
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT
    },
    **TEST_SETTINGS
)
def mustLogPretyJsonWithColors() :
    # Arrange
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass}): {type(MyClass)}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass}).__name__: {type(MyClass).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass().myMethod}): {type(MyClass().myMethod)}')
    # log.log(mustLogPretyPythonWithColors, f'type({MyClass().myMethod}).__name__: {type(MyClass().myMethod).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({myFunction}): {type(myFunction)}')
    # log.log(mustLogPretyPythonWithColors, f'type({myFunction}).__name__: {type(myFunction).__name__}')
    # log.log(mustLogPretyPythonWithColors, f'type({log}): {type(log)}')
    # log.log(mustLogPretyPythonWithColors, f'type({log}).__name__: {type(log).__name__}')
    dictionaryInstance = {
        **{
            'class' : MyClass,
            'method' : MyClass().myMethod,
            'value' : MyClass().myMethod(),
            'function' : myFunction,
            'otherValue' : myFunction(1.1),
            'module' : log
        },
        **DICTIONARY_INSTANCE
    }
    exception = None

    # Act
    try :
        log.prettyJson(mustLogPretyJsonWithColors, 'prettyJson', dictionaryInstance)
    except Exception as e :
        log.failure(mustLogPretyJsonWithColors, 'Failed to log prety json in this method call', e)
        exception = e

    # Assert
    assert ObjectHelper.isNone(exception)

@Test(
    environmentVariables={
        log.LOG : True,
        log.SUCCESS : True,
        log.SETTING : True,
        log.DEBUG : True,
        log.WARNING : True,
        log.WRAPPER : True,
        log.FAILURE : True,
        log.ERROR : True,
        log.TEST : True,
        log.ENABLE_LOGS_WITH_COLORS : True,
        SettingHelper.ACTIVE_ENVIRONMENT : SettingHelper.LOCAL_ENVIRONMENT
    },
    **TEST_SETTINGS
)
def mustPrintMessageLog_withColors() :
    # Arrange
    mustLogWithNewLine = 'must log with new line'
    mustNotLogWithNewLine = 'must not log with new line'
    mustLogWithoutNewLine = 'must log without new line'
    mustNotLogWithoutNewLine = 'must not log without new line'
    mustLogWithNewLineWithException = 'must log with new line with exception'
    mustNotLogWithNewLineWithException = 'must not log with new line with exception'
    mustLogWithoutNewLineWithException = 'must log without new line with exception'
    mustNotLogWithoutNewLineWithException = 'must not log without new line with exception'
    someExceptionMessage = 'some exception message'
    thrownException = None
    try :
        raise Exception(someExceptionMessage)
    except Exception as exception :
        thrownException = exception

    # Act
    log.printLog(mustLogWithNewLine, condition=True, newLine=True)
    log.printInfo(mustLogWithNewLine, condition=True, newLine=True)
    log.printStatus(mustLogWithNewLine, condition=True, newLine=True)
    log.printSuccess(mustLogWithNewLine, condition=True, newLine=True)
    log.printSetting(mustLogWithNewLine, condition=True, newLine=True)
    log.printDebug(mustLogWithNewLine, condition=True, newLine=True, exception=None)
    log.printWarning(mustLogWithNewLine, condition=True, newLine=True, exception=None)
    log.printWarper(mustLogWithNewLine, condition=True, newLine=True, exception=None)
    log.printFailure(mustLogWithNewLine, condition=True, newLine=True, exception=None)
    log.printError(mustLogWithNewLine, condition=True, newLine=True, exception=None)
    log.printTest(mustLogWithNewLine, condition=True, newLine=True, exception=None)

    log.printLog(mustNotLogWithNewLine, condition=False, newLine=True)
    log.printInfo(mustNotLogWithNewLine, condition=False, newLine=True)
    log.printStatus(mustNotLogWithNewLine, condition=False, newLine=True)
    log.printSuccess(mustNotLogWithNewLine, condition=False, newLine=True)
    log.printSetting(mustNotLogWithNewLine, condition=False, newLine=True)
    log.printDebug(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)
    log.printWarning(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)
    log.printWarper(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)
    log.printFailure(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)
    log.printError(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)
    log.printTest(mustNotLogWithNewLine, condition=False, newLine=True, exception=None)

    log.printLog(mustLogWithoutNewLine, condition=True, newLine=False)
    log.printInfo(mustLogWithoutNewLine, condition=True, newLine=False)
    log.printStatus(mustLogWithoutNewLine, condition=True, newLine=False)
    log.printSuccess(mustLogWithoutNewLine, condition=True, newLine=False)
    log.printSetting(mustLogWithoutNewLine, condition=True, newLine=False)
    log.printDebug(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)
    log.printWarning(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)
    log.printWarper(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)
    log.printFailure(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)
    log.printError(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)
    log.printTest(mustLogWithoutNewLine, condition=True, newLine=False, exception=None)

    log.printLog(mustNotLogWithoutNewLine, condition=False, newLine=False)
    log.printInfo(mustNotLogWithoutNewLine, condition=False, newLine=False)
    log.printStatus(mustNotLogWithoutNewLine, condition=False, newLine=False)
    log.printSuccess(mustNotLogWithoutNewLine, condition=False, newLine=False)
    log.printSetting(mustNotLogWithoutNewLine, condition=False, newLine=False)
    log.printDebug(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)
    log.printWarning(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)
    log.printWarper(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)
    log.printFailure(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)
    log.printError(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)
    log.printTest(mustNotLogWithoutNewLine, condition=False, newLine=False, exception=None)

    log.printLog(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printDebug(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printWarning(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printWarper(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printFailure(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printError(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)
    log.printTest(mustLogWithNewLineWithException, condition=True, newLine=True, exception=thrownException)

    log.printLog(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printDebug(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printWarning(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printWarper(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printFailure(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printError(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)
    log.printTest(mustLogWithoutNewLineWithException, condition=True, newLine=False, exception=thrownException)

    log.printLog(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printDebug(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printWarning(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printWarper(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printFailure(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printError(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)
    log.printTest(mustNotLogWithNewLineWithException, condition=False, newLine=True, exception=thrownException)

    log.printLog(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printDebug(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printWarning(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printWarper(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printFailure(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printError(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)
    log.printTest(mustNotLogWithoutNewLineWithException, condition=False, newLine=False, exception=thrownException)

    # Assert
    assert True == SettingHelper.activeEnvironmentIsLocal()
    assert SettingHelper.LOCAL_ENVIRONMENT == EnvironmentHelper.get(SettingHelper.ACTIVE_ENVIRONMENT)
    assert SettingHelper.LOCAL_ENVIRONMENT == SettingHelper.getActiveEnvironment()



from python_helper.api.src.domain import Constant as c
from python_helper.api.src.service import LogHelper, ReflectionHelper, ObjectHelper, SettingHelper
from python_helper.api.src.annotation.EnvironmentAnnotation import EnvironmentVariable

IS_TEST_METHOD = 'isTestMethod'

CALL_BEFORE = 'callBefore'
ARGS_OF_CALL_BEFORE = 'argsOfCallBefore'
KWARGS_OF_CALL_BEFORE = 'kwargsOfCallBefore'
CALL_AFTER = 'callAfter'
ARGS_OF_CALL_AFTER = 'argsOfCallAfter'
KWARGS_OF_CALL_AFTER = 'kwargsOfCallAfter'
RETURN_VALUE_FROM_CALL_BEFORE = 'returnOfCallBefore'
RETURN_VALUE_FROM_CALL_AFTER = 'returnOfCallAfter'

TEST_VALUE_NOT_SET = '__TEST_VALUE_NOT_SET__'
BEFORE_THE_TEST = 'before'
AFTER_THE_TEST = 'after'

def Test(
    *outerArgs,
    environmentVariables = None,
    callBefore = TEST_VALUE_NOT_SET,
    argsOfCallBefore = TEST_VALUE_NOT_SET,
    kwargsOfCallBefore = TEST_VALUE_NOT_SET,
    callAfter = TEST_VALUE_NOT_SET,
    argsOfCallAfter = TEST_VALUE_NOT_SET,
    kwargsOfCallAfter = TEST_VALUE_NOT_SET,
    returns = None,
    inspectGlobals = False,
    logResult = False,
    **outerKwargs
) :
    def innerMethodWrapper(resourceInstanceMethod,*innerMethodArgs,**innerMethodKwargs) :
        @EnvironmentVariable(environmentVariables=environmentVariables)
        def innerResourceInstanceMethod(*innerArgs,**innerKwargs) :
            handlerException = None
            handlerReturn = None
            inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'will impartialy observe')
            try :
                methodReturnException = None
                methodReturn = TEST_VALUE_NOT_SET
                handleBefore(resourceInstanceMethod, callBefore, argsOfCallBefore, kwargsOfCallBefore, returns, inspectGlobals)
                originalEnvironmentVariables, originalActiveEnvironment = SettingHelper.extractEnvironmentVariables(environmentVariables)
                inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'is about to run')
                try :
                    methodReturn = resourceInstanceMethod(*innerArgs,**innerKwargs)
                except Exception as exception :
                    methodReturnException = exception
                inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'just run')
                SettingHelper.recoverEnvironmentVariables(environmentVariables, originalEnvironmentVariables, originalActiveEnvironment)
                handlerReturn = handleAfter(resourceInstanceMethod, callAfter, argsOfCallAfter, kwargsOfCallAfter, returns, methodReturn, inspectGlobals, methodReturnException=methodReturnException, logResult=logResult)
            except Exception as exception :
                handlerException = exception
            inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'impartialy observed')
            if ObjectHelper.isNotNone(handlerException) :
                raise handlerException
            return handlerReturn
        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
        ReflectionHelper.setAttributeOrMethod(innerResourceInstanceMethod, IS_TEST_METHOD, True)
        return innerResourceInstanceMethod
    return innerMethodWrapper

def handleBefore(resourceInstanceMethod, actionClass, args, kwargs, returns, inspectGlobals) :
    LogHelper.test(resourceInstanceMethod, 'Test started')
    actionHandlerException = handle(resourceInstanceMethod, actionClass, args, kwargs, returns, BEFORE_THE_TEST, RETURN_VALUE_FROM_CALL_BEFORE)
    if ObjectHelper.isNotNone(actionHandlerException) :
        raise actionHandlerException
    inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'will run')

def handleAfter(resourceInstanceMethod, actionClass, args, kwargs, returns, methodReturn, inspectGlobals, methodReturnException=None, logResult=True) :
    inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, 'did run')
    if ObjectHelper.isNone(methodReturnException) :
        LogHelper.printSuccess(f'{ReflectionHelper.getMethodModuleNameDotName(resourceInstanceMethod)} test succeed', condition=logResult, newLine=False, margin=False)
    else :
        LogHelper.printError(f'{ReflectionHelper.getMethodModuleNameDotName(resourceInstanceMethod)} test failed', condition=logResult, newLine=False, margin=False, exception=methodReturnException)
    actionHandlerException = handle(resourceInstanceMethod, actionClass, args, kwargs, returns, AFTER_THE_TEST, RETURN_VALUE_FROM_CALL_AFTER)
    LogHelper.test(resourceInstanceMethod, 'Test completed')
    if ObjectHelper.isNotNone(methodReturnException) or ObjectHelper.isNotNone(actionHandlerException) :
        if ObjectHelper.isNotNone(methodReturnException) and ObjectHelper.isNotNone(actionHandlerException) :
            raise Exception(f'{LogHelper.getExceptionMessage(methodReturnException)}. Followed by: {LogHelper.getExceptionMessage(actionHandlerException)}')
        elif ObjectHelper.isNotNone(methodReturnException) :
            raise methodReturnException
        raise actionHandlerException
    if not TEST_VALUE_NOT_SET == methodReturn :
        return methodReturn

def handle(resourceInstanceMethod, actionClass, args, kwargs, returns, moment, returnKey) :
    try :
        returnCall = None
        if actionIsPresent(actionClass) :
            argsMessage = getArgsLogMessage(args)
            kwargsMessage = getKwargsLogMessage(kwargs)
            returnCall = getRetrunValue(actionClass, args, kwargs)
            LogHelper.test(resourceInstanceMethod, f'{ReflectionHelper.getName(actionClass)}({argsMessage}, {kwargsMessage}): {returnCall}')
        else :
            LogHelper.test(resourceInstanceMethod, f'Test handler to perform actions {moment} the test is not defined')
        if returnsValueIsPresent(returns) :
            returns[returnKey] = returnCall
    except Exception as exception :
        LogHelper.printError(f'{ReflectionHelper.getMethodModuleNameDotName(resourceInstanceMethod)} test went wrong while handling actions {moment} the test. *args: {args}, **kwargs: {kwargs}. Enable test logs for more information', condition=True, newLine=False, margin=False, exception=exception)
        return exception

def getArgsLogMessage(args) :
    if TEST_VALUE_NOT_SET == args :
        return '*()'
    return f'*({args})'

def getKwargsLogMessage(kwargs) :
    if TEST_VALUE_NOT_SET == kwargs :
        return '**{}'
    return f'**{kwargs}'

def getRetrunValue(actionClass, givenArgs, givenKwargs) :
    if actionIsPresent(actionClass) :
        args = [] if TEST_VALUE_NOT_SET == givenArgs else givenArgs
        kwargs = {} if TEST_VALUE_NOT_SET == givenKwargs else givenKwargs
        return actionClass(*args, **kwargs)

def actionIsPresent(actionClass) :
    return not TEST_VALUE_NOT_SET == actionClass and ObjectHelper.isNotNone(actionClass)

def returnsValueIsPresent(returns) :
    isPresent = ObjectHelper.isDictionary(returns)
    if not isPresent :
        LogHelper.test(returnsValueIsPresent, f'the key "returns" from {ReflectionHelper.getMethodModuleNameDotName(Test)} annotation call was not defined')
    return isPresent

def inspectGlobalsIfNeeded(inspectGlobals, resourceInstanceMethod, tense) :
    if inspectGlobals :
        import globals
        LogHelper.printDebug(f'''Inspection: {globals.getGlobalsInstance(muteLogs=inspectGlobals)} globals instance {tense} {ReflectionHelper.getMethodModuleNameDotName(resourceInstanceMethod)}''', condition=inspectGlobals, newLine=False, margin=False)



from distutils.core import setup
import os

OS_SEPARATOR = os.path.sep

version = '0.3.41'
name = 'python_helper'
url = f'https://github.com/SamuelJansen/{name}/'

setup(
    name = name,
    packages = [
        name,
        f'{name}{OS_SEPARATOR}api',
        f'{name}{OS_SEPARATOR}api{OS_SEPARATOR}src',
        f'{name}{OS_SEPARATOR}api{OS_SEPARATOR}src{OS_SEPARATOR}service',
        f'{name}{OS_SEPARATOR}api{OS_SEPARATOR}src{OS_SEPARATOR}domain',
        f'{name}{OS_SEPARATOR}api{OS_SEPARATOR}src{OS_SEPARATOR}helper',
        f'{name}{OS_SEPARATOR}api{OS_SEPARATOR}src{OS_SEPARATOR}annotation'
    ],
    version = version,
    license = 'MIT',
    description = 'python helper package',
    author = 'Samuel Jansen',
    author_email = 'samuel.jansenn@gmail.com',
    url = url,
    download_url = f'{url}archive/v{version}.tar.gz',
    keywords = ['helper', 'python helper package', 'python helper', 'helper package'],
    install_requires = [
        'colorama==0.4.4'
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7'
    ],
    python_requires='>=3.7'
)



from python_helper.api.src.domain import Constant as c
from python_helper.api.src.service import LogHelper


def getFileLines(filePath: str, encoding: str = c.UTF_8):
    lines = []
    try:
        with open(filePath, c.READ, encoding=encoding) as file :
            lines = file.readlines()
    except Exception as exception:
        LogHelper.error(getFileLines, 'Not possible to read lines', exception, muteStackTrace=True)
        raise exception
    return lines



from python_helper.api.src.service import LogHelper as log
from python_helper.api.src.domain import Constant
from python_helper.api.src.service import ObjectHelper, SettingHelper, StringHelper, EnvironmentHelper, ReflectionHelper, RandomHelper, DateTimeHelper, FileHelper
from python_helper.api.src.helper import ObjectHelperHelper, SettingHelperHelper, SettingHelperHelper, LogHelperHelper, RandomHelperHelper
from python_helper.api.src.annotation.EnvironmentAnnotation import EnvironmentVariable
from python_helper.api.src.annotation.MethodAnnotation import Method, Function
from python_helper.api.src.annotation import TestAnnotation
from python_helper.api.src.annotation.TestAnnotation import Test
from python_helper.api.src.service import TestHelper



import colorama, traceback
from python_helper.api.src.domain import Constant as c
from python_helper.api.src.service import SettingHelper, StringHelper, EnvironmentHelper, ObjectHelper, ReflectionHelper

LOG = 'LOG'
INFO = 'INFO'
STATUS = 'STATUS'
SUCCESS = 'SUCCESS'
SETTING = 'SETTING'
DEBUG = 'DEBUG'
WARNING = 'WARNING'
WRAPPER = 'WRAPPER'
FAILURE = 'FAILURE'
ERROR = 'ERROR'
TEST = 'TEST'

from python_helper.api.src.helper import LogHelperHelper

RESET_ALL_COLORS = colorama.Style.RESET_ALL
COLOR_SET = set([*[cv for dv in LogHelperHelper.LEVEL_DICTIONARY.values() for ck, cv in dv.items() if ck in [LogHelperHelper.FIRST_LAYER_COLOR, LogHelperHelper.SECOND_LAYER_COLOR]], c.RESET_COLOR])

LOGS_FILE_NAME = 'LOGS_FILE_NAME'
ENABLE_LOGS_WITH_COLORS = 'ENABLE_LOGS_WITH_COLORS'
LOGS_WITH_COLORS = 'LOGS_WITH_COLORS'


LOG_LEVEL_ENABLED_BY_DEFAULT_LIST = (
    ERROR
    , SUCCESS
    # , FAILURE
    #,  SETTING
    # , STATUS
    # , INFO
    # , WARNING
    # , DEBUG
    # , WRAPPER
    # , LOG
    # , TEST
)


global LOG_HELPER_SETTINGS


def colorsEnabled(enable=False):
    return enable or EnvironmentHelper.isTrue(ENABLE_LOGS_WITH_COLORS, default=False) or SettingHelper.activeEnvironmentIsLocal()

def logIt(it, **kwargs):
    # if not (it in LogHelperHelper.COLOR_SET):
    #     print(it, **kwargs)
    print(it, **kwargs)

def loadSettings(logsFileName=None, withColors=False, enabledByDefault=LOG_LEVEL_ENABLED_BY_DEFAULT_LIST) :
    global LOG_HELPER_SETTINGS
    colorama.deinit()
    activeEnvironment = SettingHelper.getActiveEnvironment()
    colorsAreEnabled = withColors or colorsEnabled(enable=activeEnvironment==SettingHelper.LOCAL_ENVIRONMENT)
    LOG_HELPER_SETTINGS = {
        **{
            SettingHelper.ACTIVE_ENVIRONMENT: activeEnvironment,
            LOGS_FILE_NAME: logsFileName,
            LOGS_WITH_COLORS: colorsAreEnabled
        },
        **{
            level: (c.TRUE if EnvironmentHelper.isTrue(level, default=False) or level in enabledByDefault else c.FALSE) for level in LogHelperHelper.LEVEL_DICTIONARY
        }
    }
    if colorsAreEnabled:
        colorama.init()

    # global LOG_HELPER_SETTINGS
    # colorama.deinit()
    # settings = {}
    # settings[SettingHelper.ACTIVE_ENVIRONMENT] = SettingHelper.getActiveEnvironment()
    # for level in LogHelperHelper.LEVEL_DICTIONARY :
    #     settings[level] = c.TRUE if EnvironmentHelper.isTrue(level, default=True) else c.FALSE
    # LOG_HELPER_SETTINGS = settings
    # if colorsEnabled():
    #     colorama.init()
    #     # logIt(RESET_ALL_COLORS, end=c.BLANK)

loadSettings()

def log(origin, message, exception=None, muteStackTrace=False, newLine=False, level=LOG) :
    LogHelperHelper.softLog(origin, message, level, muteStackTrace=muteStackTrace, newLine=newLine, exception=exception)

def info(origin, message, newLine=False) :
    LogHelperHelper.softLog(origin, message, INFO, muteStackTrace=True, newLine=newLine)

def status(origin, message, newLine=False) :
    LogHelperHelper.softLog(origin, message, STATUS, muteStackTrace=True, newLine=newLine)

def success(origin, message, newLine=False) :
    LogHelperHelper.softLog(origin, message, SUCCESS, muteStackTrace=True, newLine=newLine)

def setting(origin, message, muteStackTrace=False, newLine=False) :
    LogHelperHelper.softLog(origin, message, SETTING, muteStackTrace=muteStackTrace, newLine=newLine)

def debug(origin, message, exception=None, muteStackTrace=False, newLine=False) :
    LogHelperHelper.softLog(origin, message, DEBUG, muteStackTrace=muteStackTrace, newLine=newLine, exception=exception)

def warning(origin, message, exception=None, muteStackTrace=False, newLine=False) :
    LogHelperHelper.softLog(origin, message, WARNING, muteStackTrace=muteStackTrace, newLine=newLine, exception=exception)

def wrapper(origin, message, exception=None, muteStackTrace=False, newLine=False) :
    LogHelperHelper.softLog(origin, message, WRAPPER, muteStackTrace=muteStackTrace, newLine=newLine, exception=exception)

def failure(origin, message, exception, muteStackTrace=False, newLine=False) :
    LogHelperHelper.hardLog(origin, message, exception, FAILURE, muteStackTrace=muteStackTrace, newLine=newLine)

def error(origin, message, exception, muteStackTrace=False, newLine=False) :
    LogHelperHelper.hardLog(origin, message, exception, ERROR, muteStackTrace=muteStackTrace, newLine=newLine)

def test(origin, message, exception=None, muteStackTrace=False, newLine=False) :
    LogHelperHelper.softLog(origin, message, TEST, muteStackTrace=muteStackTrace, newLine=newLine, exception=exception)

def printLog(message, level=LOG, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(level, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printInfo(message, condition=False, newLine=True, margin=True) :
    LogHelperHelper.printMessageLog(INFO, message, condition=condition, muteStackTrace=True, newLine=newLine, margin=margin)

def printStatus(message, condition=False, newLine=True, margin=True) :
    LogHelperHelper.printMessageLog(STATUS, message, condition=condition, muteStackTrace=True, newLine=newLine, margin=margin)

def printSuccess(message, condition=False, newLine=True, margin=True) :
    LogHelperHelper.printMessageLog(SUCCESS, message, condition=condition, muteStackTrace=True, newLine=newLine, margin=margin)

def printSetting(message, condition=False, muteStackTrace=False, newLine=True, margin=True) :
    LogHelperHelper.printMessageLog(SETTING, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin)

def printDebug(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(DEBUG, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printWarning(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(WARNING, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printWarper(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(WRAPPER, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printFailure(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(FAILURE, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printError(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(ERROR, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def printTest(message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    LogHelperHelper.printMessageLog(TEST, message, condition=condition, muteStackTrace=muteStackTrace, newLine=newLine, margin=margin, exception=exception)

def prettyPython(
        origin,
        message,
        dictionaryInstance,
        quote = c.SINGLE_QUOTE,
        tabCount = 0,
        nullValue = c.NONE,
        trueValue = c.TRUE,
        falseValue = c.FALSE,
        logLevel = LOG,
        condition = True
    ) :
    if condition :
        stdout, stderr = EnvironmentHelper.getCurrentSoutStatus()
        prettyPythonValue = StringHelper.prettyPython(
            dictionaryInstance,
            quote = quote,
            tabCount = tabCount,
            nullValue = nullValue,
            trueValue = trueValue,
            falseValue = falseValue,
            withColors = colorsEnabled(),
            joinAtReturn = False
        )
        LogHelperHelper.softLog(origin, StringHelper.join([message, c.COLON_SPACE, *prettyPythonValue]), logLevel)
        EnvironmentHelper.overrideSoutStatus(stdout, stderr)

def prettyJson(
        origin,
        message,
        dictionaryInstance,
        quote = c.DOUBLE_QUOTE,
        tabCount = 0,
        nullValue = c.NULL_VALUE,
        trueValue = c.TRUE_VALUE,
        falseValue = c.FALSE_VALUE,
        logLevel = LOG,
        condition = True
    ) :
    if condition :
        stdout, stderr = EnvironmentHelper.getCurrentSoutStatus()
        prettyJsonValue = StringHelper.prettyJson(
            dictionaryInstance,
            quote = quote,
            tabCount = tabCount,
            nullValue = nullValue,
            trueValue = trueValue,
            falseValue = falseValue,
            withColors = colorsEnabled(),
            joinAtReturn = False
        )
        LogHelperHelper.softLog(origin, StringHelper.join([message, c.COLON_SPACE, *prettyJsonValue]), logLevel)
        EnvironmentHelper.overrideSoutStatus(stdout, stderr)

def getExceptionMessage(exception) :
    if ObjectHelper.isEmpty(exception) :
        return c.UNKNOWN
    exceptionAsString = str(exception)
    if c.BLANK == exceptionAsString :
        return ReflectionHelper.getName(exception.__class__)
    else :
        return exceptionAsString

def getTracebackMessage(muteStackTrace) :
    tracebackMessage = c.BLANK
    try :
        tracebackMessage = traceback.format_exc()
    except :
        tracebackMessage = f'{c.NEW_LINE}'
    if muteStackTrace :
        return StringHelper.join(tracebackMessage.split(c.NEW_LINE)[-2:], character=c.NEW_LINE)
    return LogHelperHelper.NO_TRACEBACK_PRESENT_MESSAGE if LogHelperHelper.NO_TRACEBACK_PRESENT == str(tracebackMessage) else tracebackMessage



import os, sys, json
from python_helper.api.src.service import StringHelper, LogHelper, ObjectHelper, SettingHelper
from python_helper.api.src.domain import Constant as c

OS = os
SYS = sys
OS_SEPARATOR = OS.path.sep

LINUX_OS_NAME = 'linux'
WINDOWS_OS_NAME = 'win32'

clear = lambda: OS.system('cls')

def get(environmentKey, default=None) :
    environmentValue = default if ObjectHelper.isNone(environmentKey) else OS.environ.get(environmentKey)
    return environmentValue if ObjectHelper.isNotNone(environmentValue) else default

def update(environmentKey, environmentValue, default=None) :
    if ObjectHelper.isNotEmpty(environmentKey) :
        associatedValue = None
        if ObjectHelper.isNotNone(environmentValue):
            associatedValue = str(StringHelper.filterString(environmentValue))
            OS.environ[environmentKey] = associatedValue
        elif ObjectHelper.isNotNone(default):
            associatedValue = str(StringHelper.filterString(default))
            OS.environ[environmentKey] = associatedValue
        else :
            try:
                delete(environmentKey)
            except Exception as exception :
                LogHelper.warning(update, f'Failed to delete enviroment variable key "{environmentKey}" while updating it to "{environmentValue}"', exception=exception)
        return associatedValue
    else :
        LogHelper.debug(update, f'arguments: environmentKey: {environmentKey}, environmentValue: {environmentValue}, default: {default}')
        raise Exception(f'Error associating environment variable "{environmentKey}" key to environment variable "{environmentValue}" value')

def switch(environmentKey, environmentValue, default=None) :
    originalEnvironmentValue = get(environmentKey, default=default)
    update(environmentKey, environmentValue, default=default)
    return originalEnvironmentValue

def reset(environmentVariables, originalEnvironmentVariables) :
    if environmentVariables :
        for key in environmentVariables.keys() :
            if key in originalEnvironmentVariables :
                update(key, originalEnvironmentVariables[key])

def delete(environmentKey) :
    if ObjectHelper.isNotNone(environmentKey) :
        OS.environ.pop(environmentKey)

def getSet(avoidRecursiveCall=False) :
    try :
        return {key : OS.environ[key] for key in OS.environ}
    except Exception as exception :
        LogHelper.error(getSet, 'Not possible to load os.environ as a json. Returning os.environ as string by default', exception)
        return str(OS.environ)[8:-1]

def isNone(environmentKey, default=True, evaluateItInsted=None):
    if ObjectHelper.isNotNone(environmentKey) :
        return c.NONE == (
            evaluateItInsted if ObjectHelper.isNotNone(evaluateItInsted) else get(environmentKey, default=c.NONE)
        )
    return default

def isNotNone(environmentKey, default=True, evaluateItInsted=None):
    if ObjectHelper.isNotNone(environmentKey) :
        return not c.NONE == (
            evaluateItInsted if ObjectHelper.isNotNone(evaluateItInsted) else get(environmentKey, default=c.NONE)
        )
    return default

def isBoolean(environmentKey, default=False, evaluateItInsted=None):
    if ObjectHelper.isNotNone(environmentKey) :
        if ObjectHelper.isNotNone(evaluateItInsted):
            return evaluateItInsted in [c.TRUE, c.FALSE] if isNotNone(environmentKey, evaluateItInsted=evaluateItInsted) else default
        return get(environmentKey, default=default) in [c.TRUE, c.FALSE]
    return default

def isTrue(environmentKey, default=False, evaluateItInsted=None):
    if ObjectHelper.isNotNone(environmentKey) :
        if ObjectHelper.isNotNone(evaluateItInsted):
            return c.TRUE == evaluateItInsted if isBoolean(environmentKey, default=default, evaluateItInsted=evaluateItInsted) else default
        innerEvaluatItInsted = get(environmentKey, default=None)
        if ObjectHelper.isNotNone(innerEvaluatItInsted):
            return isTrue(environmentKey, default=default, evaluateItInsted=innerEvaluatItInsted)
        if isBoolean(environmentKey, evaluateItInsted=innerEvaluatItInsted):
            return c.TRUE == c.TRUE if isinstance(default, bool) and default else c.FALSE
    return default

def isFalse(environmentKey, default=True, evaluateItInsted=None):
    # if ObjectHelper.isNotNone(environmentKey) :
    #     innerEvaluatItInsted = get(environmentKey, default=None)
    #     if ObjectHelper.isNotNone(evaluateItInsted):
    #         return c.FALSE == evaluateItInsted if isBoolean(environmentKey, evaluateItInsted=innerEvaluatItInsted) else default
    #     if isBoolean(environmentKey):
    #         return c.FALSE == c.FALSE if isinstance(default, bool) and not default else c.TRUE
    # return default


    if ObjectHelper.isNotNone(environmentKey) :
        if ObjectHelper.isNotNone(evaluateItInsted):
            return c.FALSE == evaluateItInsted if isBoolean(environmentKey, default=default, evaluateItInsted=evaluateItInsted) else default
        innerEvaluatItInsted = get(environmentKey, default=None)
        if ObjectHelper.isNotNone(innerEvaluatItInsted):
            return isTrue(environmentKey, default=default, evaluateItInsted=innerEvaluatItInsted)
        if isBoolean(environmentKey, evaluateItInsted=innerEvaluatItInsted):
            return c.FALSE == c.FALSE if isinstance(default, bool) and not default else c.TRUE
    return default

def listDirectoryContent(path) :
    return OS.listdir(path)

def isDirectory(path) :
    return OS.path.isdir(path)

def isFile(path) :
    return OS.path.isfile(path)

def makeDirectory(path, accessRights=0o777) :
    ###- accessRights = 0o777 --> write, access and read
    ###- accessRights = 0o755 --> access and read
    ###- accessRights = 0o755 --> access and read
    OS.makedirs(path, mode=accessRights)

def appendPath(path) :
    SYS.path.append(path)

def getCurrentSoutStatus(avoidRecursiveCall=False) :
    return SYS.stdout, SYS.stderr

def overrideSoutStatus(stdout, stderr) :
    SYS.stdout = stdout
    SYS.stderr = stderr

def isLinux() :
    return SYS.platform == LINUX_OS_NAME

def isWindows() :
    return SYS.platform == WINDOWS_OS_NAME



from python_helper.api.src.domain import Constant as c
from python_helper.api.src.service import LogHelper, ObjectHelper, ReflectionHelper, StringHelper

NO_TRACEBACK_PRESENT = f'NoneType: None{c.NEW_LINE}'
NO_TRACEBACK_PRESENT_MESSAGE = 'Exception: '

FIRST_LAYER_COLOR = 'FIRST_LAYER_COLOR'
SECOND_LAYER_COLOR = 'SECOND_LAYER_COLOR'
LOG_TEXT = 'LOG_TEXT'

LEVEL_DICTIONARY = {
    LogHelper.LOG : {
        FIRST_LAYER_COLOR : c.BRIGHT_BLACK,
        SECOND_LAYER_COLOR: c.BRIGHT_BLACK,
        LOG_TEXT : c.LOG
    },
    LogHelper.INFO : {
        FIRST_LAYER_COLOR : c.BRIGHT_BLACK,
        SECOND_LAYER_COLOR : c.DARK_WHITE,
        LOG_TEXT : c.INFO
    },
    LogHelper.STATUS : {
        FIRST_LAYER_COLOR : c.BRIGHT_GREEN,
        SECOND_LAYER_COLOR : c.BRIGHT_WHITE,
        LOG_TEXT : c.STATUS
    },
    LogHelper.SUCCESS : {
        FIRST_LAYER_COLOR : c.DARK_GREEN,
        SECOND_LAYER_COLOR: c.BRIGHT_GREEN,
        LOG_TEXT : c.SUCCESS
    },
    LogHelper.SETTING : {
        FIRST_LAYER_COLOR : c.DARK_BLUE,
        SECOND_LAYER_COLOR: c.BRIGHT_BLUE,
        LOG_TEXT : c.SETTING
    },
    LogHelper.DEBUG : {
        FIRST_LAYER_COLOR : c.DARK_CYAN,
        SECOND_LAYER_COLOR: c.BRIGHT_CYAN,
        LOG_TEXT : c.DEBUG
    },
    LogHelper.WARNING : {
        FIRST_LAYER_COLOR : c.DARK_YELLOW,
        SECOND_LAYER_COLOR: c.BRIGHT_YELLOW,
        LOG_TEXT : c.WARNING
    },
    LogHelper.WRAPPER : {
        FIRST_LAYER_COLOR : c.BRIGHT_WHITE,
        SECOND_LAYER_COLOR: c.DARK_WHITE,
        LOG_TEXT : c.WRAPPER
    },
    LogHelper.FAILURE : {
        FIRST_LAYER_COLOR : c.DARK_MAGENTA,
        SECOND_LAYER_COLOR: c.BRIGHT_MAGENTA,
        LOG_TEXT : c.FAILURE
    },
    LogHelper.ERROR : {
        FIRST_LAYER_COLOR : c.DARK_RED,
        SECOND_LAYER_COLOR: c.BRIGHT_RED,
        LOG_TEXT : c.ERROR
    },
    LogHelper.TEST : {
        FIRST_LAYER_COLOR : c.BRIGHT_BLACK,
        SECOND_LAYER_COLOR: c.BRIGHT_BLACK,
        LOG_TEXT : c.TEST
    }
}


def getStatus(level) :
    status = LogHelper.LOG_HELPER_SETTINGS.get(level)
    return status if isinstance(status, str) and StringHelper.isNotBlank(status) else c.TRUE

def getColors(level) :
    if LogHelper.colorsEnabled():
        firstLayerColor = LEVEL_DICTIONARY.get(level).get(FIRST_LAYER_COLOR) if LEVEL_DICTIONARY.get(level) and LEVEL_DICTIONARY.get(level).get(FIRST_LAYER_COLOR) else c.BLANK
        secondLayerColor = LEVEL_DICTIONARY.get(level).get(SECOND_LAYER_COLOR) if LEVEL_DICTIONARY.get(level) and LEVEL_DICTIONARY.get(level).get(SECOND_LAYER_COLOR) else c.BLANK
        tirdLayerColor = c.MUTTED_COLOR if c.MUTTED_COLOR else c.BLANK
        resetColor = c.RESET_COLOR if c.RESET_COLOR else c.BLANK
    else :
        firstLayerColor = c.BLANK
        secondLayerColor = c.BLANK
        tirdLayerColor = c.BLANK
        resetColor = c.BLANK
    return (firstLayerColor, secondLayerColor, tirdLayerColor, resetColor) if StringHelper.isNotBlank(firstLayerColor) else (c.BLANK, c.BLANK, c.BLANK, c.BLANK)

def softLog(origin, message, level, exception=None, muteStackTrace=False, newLine=False) :
    if ObjectHelper.isNotNone(exception) :
        hardLog(origin, message, exception, level, muteStackTrace=muteStackTrace)
    elif c.TRUE == getStatus(level) :
        firstLayerColor, secondLayerColor, tirdLayerColor, resetColor = getColors(level)
        LogHelper.logIt(StringHelper.join([firstLayerColor, LEVEL_DICTIONARY[level][LOG_TEXT], *getOriginPortion(origin, tirdLayerColor, resetColor), secondLayerColor, message, resetColor, getNewLine(newLine, exception=exception, muteStackTrace=muteStackTrace)]))
    elif not c.FALSE == getStatus(level) :
        levelStatusError(method, level)

def hardLog(origin, message, exception, level, muteStackTrace=False, newLine=False) :
    if c.TRUE == getStatus(level) :
        firstLayerColor, secondLayerColor, tirdLayerColor, resetColor = getColors(level)
        LogHelper.logIt(StringHelper.join([firstLayerColor, LEVEL_DICTIONARY[level][LOG_TEXT], *getOriginPortion(origin, tirdLayerColor, resetColor), secondLayerColor, message, *getErrorPortion(exception, muteStackTrace, firstLayerColor, secondLayerColor, tirdLayerColor, resetColor), resetColor, getNewLine(newLine, exception=exception, muteStackTrace=muteStackTrace)]))
    elif not c.FALSE == getStatus(level) :
        levelStatusError(method, level)

def printMessageLog(level, message, condition=False, muteStackTrace=False, newLine=True, margin=True, exception=None) :
    if condition :
        firstLayerColor, secondLayerColor, tirdLayerColor, resetColor = getColors(level)
        LogHelper.logIt(StringHelper.join([c.TAB if margin else c.BLANK, firstLayerColor, LEVEL_DICTIONARY[level][LOG_TEXT], secondLayerColor, message, *getErrorPortion(exception, muteStackTrace, firstLayerColor, secondLayerColor, tirdLayerColor, resetColor), resetColor, getNewLine(newLine, exception=exception, muteStackTrace=muteStackTrace)]))

def getOriginPortion(origin, tirdLayerColor, resetColor) :
    if not origin or origin == c.BLANK :
        return [c.BLANK]
    else :
        moduleName = ReflectionHelper.getModuleName(origin)
        className = ReflectionHelper.getClassName(origin)
        moduleProtion = getLogThingName(moduleName)
        classPortion = getLogThingName(className)
        return [tirdLayerColor, *moduleProtion, *classPortion, ReflectionHelper.getName(origin), c.COLON_SPACE, resetColor]

def getLogThingName(thing) :
    return [] if thing in c.NATIVE_TYPES or (c.OPEN_TUPLE in thing and c.CLOSE_TUPLE in thing) else [thing, c.DOT]

def getErrorPortion(exception, muteStackTrace, firstLayerColor, secondLayerColor, tirdLayerColor, resetColor) :
    if ObjectHelper.isEmpty(exception) :
        return [c.BLANK]
    exceptionMessage = LogHelper.getExceptionMessage(exception)
    traceBackMessage = LogHelper.getTracebackMessage(muteStackTrace)
    traceBackMessageSplited = traceBackMessage.split(exceptionMessage)
    return [c.NEW_LINE, tirdLayerColor, *[t if t is not traceBackMessageSplited[-1] else t if t[-1] is not c.NEW_LINE else t[:-1] for t in traceBackMessageSplited if ObjectHelper.isNotNone(t)], secondLayerColor, exceptionMessage, resetColor]

def levelStatusError(method, level) :
    LogHelper.failure(method, f'"{level}" log level status is not properly defined: {getStatus(level)}', None)

def getNewLine(newLine, exception=None, muteStackTrace=False) :
    return c.NEW_LINE if (newLine and ObjectHelper.isNone(exception)) or (ObjectHelper.isNotNone(exception) and NO_TRACEBACK_PRESENT_MESSAGE == LogHelper.getTracebackMessage(muteStackTrace)) else c.BLANK

# FORE_SIMPLE_RESET_COLOR = colorama.Fore.RESET
# LEVEL_DICTIONARY = {
#     SUCCESS : {
#         FIRST_LAYER_COLOR : colorama.Fore.GREEN,
#         SECOND_LAYER_COLOR: colorama.Fore.GREEN + colorama.Style.BRIGHT
#     },
#     SETTING : {
#         FIRST_LAYER_COLOR : colorama.Fore.BLUE,
#         SECOND_LAYER_COLOR: colorama.Fore.BLUE + colorama.Style.BRIGHT
#     },
#     DEBUG : {
#         FIRST_LAYER_COLOR : colorama.Fore.CYAN,
#         SECOND_LAYER_COLOR: colorama.Fore.CYAN + colorama.Style.BRIGHT
#     },
#     WARNING : {
#         FIRST_LAYER_COLOR : colorama.Fore.YELLOW,
#         SECOND_LAYER_COLOR: colorama.Fore.YELLOW + colorama.Style.BRIGHT
#     },
#     WRAPPER : {
#         FIRST_LAYER_COLOR : colorama.Fore.WHITE + colorama.Style.BRIGHT,
#         SECOND_LAYER_COLOR: colorama.Fore.WHITE
#     },
#     FAILURE : {
#         FIRST_LAYER_COLOR : colorama.Fore.MAGENTA,
#         SECOND_LAYER_COLOR: colorama.Fore.MAGENTA + colorama.Style.BRIGHT
#     },
#     ERROR : {
#         FIRST_LAYER_COLOR : colorama.Fore.RED,
#         SECOND_LAYER_COLOR: colorama.Fore.RED + colorama.Style.BRIGHT
#     }
# }

# def print_format_table():
#     """
#     prints table of formatted text format options
#     """
#     for style in range(8):
#         for fg in range(30,38):
#             s1 = ''
#             for bg in range(40,48):
#                 format = ';'.join([str(style), str(fg), str(bg)])
#                 s1 += '\x1b[%sm %s \x1b[0m' % (format, format)
#             print(s1)
#         print('\n')
#
# print_format_table()
#
# x = 0
# for i in range(24):
#   colors = ""
#   for j in range(5):
#     code = str(x+j)
#     colors = colors + "\33[" + code + "m\\33[" + code + "m\033[0m "
#   print(colors)
#   x=x+5



import time, datetime
from python_helper import EnvironmentHelper, SettingHelper, ObjectHelper, log, Test, DateTimeHelper

# LOG_HELPER_SETTINGS = {
#     log.LOG : True,
#     log.SUCCESS : True,
#     log.SETTING : True,
#     log.DEBUG : True,
#     log.WARNING : True,
#     log.WRAPPER : True,
#     log.FAILURE : True,
#     log.ERROR : True,
    # log.TEST : False
# }

LOG_HELPER_SETTINGS = {
    log.LOG : False,
    log.SUCCESS : False,
    log.SETTING : False,
    log.DEBUG : False,
    log.WARNING : False,
    log.WRAPPER : False,
    log.FAILURE : False,
    log.ERROR : False,
    log.TEST : False
}

TEST_SETTINGS = {}

@Test()
def timeDelta() :
    # arrange
    # act
    # assert
    assert datetime.timedelta(days=2, hours=3, minutes=4, seconds=5, milliseconds=6, microseconds=7) == DateTimeHelper.timeDelta(days=2, hours=3, minutes=4, seconds=5, milliseconds=6, microseconds=7)
    assert datetime.timedelta(days=0, hours=3, minutes=4, seconds=5, milliseconds=6, microseconds=7) == DateTimeHelper.timeDelta(days=0, hours=3, minutes=4, seconds=5, milliseconds=6, microseconds=7)

# @Test()
# def dateTime_utcnow() :
#     # arrange
#     givenSimpleDateTime = time.time()
#     givenDateTime = date=datetime.datetime.utcnow()
#     givenDate = date=datetime.datetime.utcnow().date()
#     givenTime = date=datetime.datetime.utcnow().time()
#
#     # act
#     # assert
#     assert datetime.datetime.utcnow() == DateTimeHelper.dateTimeNow()
#     assert datetime.datetime.utcnow().date() == DateTimeHelper.dateOf(datetime.datetime.utcnow())
#     assert datetime.datetime.utcnow().time() == DateTimeHelper.timeOf(datetime.datetime.utcnow())
#     assert datetime.datetime.utcnow().date() == DateTimeHelper.dateNow()
#     assert datetime.datetime.utcnow().time() == DateTimeHelper.timeNow()
#     assert datetime.datetime.timestamp(datetime.datetime.utcnow()) == DateTimeHelper.timestampNow()
#     assert datetime.datetime.fromtimestamp(givenSimpleDateTime) == DateTimeHelper.ofTimestamp(givenSimpleDateTime)
#     assert datetime.datetime.utcnow() == DateTimeHelper.ofTimestamp(datetime.datetime.timestamp(DateTimeHelper.dateTimeNow()))
#
#     parsed = None
#     for pattern in DateTimeHelper.PATTERNS :
#         try :
#             parsed = datetime.datetime.strptime(str(givenDateTime), pattern)
#             break
#         except Exception as exception :
#             pass
#     assert parsed == DateTimeHelper.of(dateTime=givenDateTime)
#
#     parsed = None
#     for pattern in DateTimeHelper.PATTERNS :
#         try :
#             parsed = datetime.datetime.strptime(str(givenDate), pattern)
#             break
#         except Exception as exception :
#             pass
#     assert parsed == DateTimeHelper.of(date=givenDate)
#
#     parsed = None
#     for pattern in DateTimeHelper.PATTERNS :
#         try :
#             parsed = datetime.datetime.strptime(f'{datetime.datetime.utcnow().date()} {str(givenTime)}', pattern)
#             break
#         except Exception as exception :
#             pass
#     assert parsed == DateTimeHelper.of(time=givenTime), (parsed, DateTimeHelper.of(time=givenTime))
#
#     assert datetime.datetime.timestamp(givenDateTime) == DateTimeHelper.timestampOf(dateTime=givenDateTime), (datetime.datetime.timestamp(givenDateTime), DateTimeHelper.timestampOf(dateTime=givenDateTime))

@Test()
def dateTime_now() :
    # arrange
    givenSimpleDateTime = time.time()
    givenDateTime = date=datetime.datetime.now()
    givenDate = date=datetime.datetime.now().date()
    givenTime = date=datetime.datetime.now().time()

    # act
    # assert
    assert datetime.datetime.now() == DateTimeHelper.dateTimeNow()
    assert datetime.datetime.now().date() == DateTimeHelper.dateOf(datetime.datetime.now())
    assert datetime.datetime.now().time() == DateTimeHelper.timeOf(datetime.datetime.now())
    assert datetime.datetime.now().date() == DateTimeHelper.dateNow()
    assert datetime.datetime.now().time() == DateTimeHelper.timeNow()
    assert datetime.datetime.timestamp(datetime.datetime.now()) == DateTimeHelper.timestampNow()
    assert datetime.datetime.fromtimestamp(givenSimpleDateTime) == DateTimeHelper.ofTimestamp(givenSimpleDateTime)
    assert datetime.datetime.now() == DateTimeHelper.ofTimestamp(datetime.datetime.timestamp(DateTimeHelper.dateTimeNow()))

    parsed = None
    for pattern in DateTimeHelper.PATTERNS :
        try :
            parsed = datetime.datetime.strptime(str(givenDateTime), pattern)
            break
        except Exception as exception :
            pass
    assert parsed == DateTimeHelper.of(dateTime=givenDateTime)

    parsed = None
    for pattern in DateTimeHelper.PATTERNS :
        try :
            parsed = datetime.datetime.strptime(str(givenDate), pattern)
            break
        except Exception as exception :
            pass
    assert parsed == DateTimeHelper.of(date=givenDate)

    parsed = None
    for pattern in DateTimeHelper.PATTERNS :
        try :
            parsed = datetime.datetime.strptime(f'{datetime.datetime.now().date()} {str(givenTime)}', pattern)
            break
        except Exception as exception :
            pass
    assert parsed == DateTimeHelper.of(time=givenTime), (parsed, DateTimeHelper.of(time=givenTime))

    assert datetime.datetime.timestamp(givenDateTime) == DateTimeHelper.timestampOf(dateTime=givenDateTime), (datetime.datetime.timestamp(givenDateTime), DateTimeHelper.timestampOf(dateTime=givenDateTime))

@Test()
def getWeekDay() :
    # arrange
    dateTimeNow = datetime.datetime.now()
    timeNow = datetime.datetime.now().time()
    dateNow = datetime.datetime.now().date()

    # act
    # assert
    assert datetime.datetime.now().weekday() == DateTimeHelper.getWeekDay()
    assert datetime.datetime.now().weekday() == DateTimeHelper.getWeekDay(dateTime=dateTimeNow)
    assert datetime.datetime.now().weekday() == DateTimeHelper.getWeekDay(date=dateNow, time=timeNow)
    assert datetime.datetime.now().weekday() == DateTimeHelper.getWeekDay(date=dateNow)
